//
//  PhysicalSimulator.h
//  InteractiveObjects
//

#ifndef __Test__PhysicalSimulator__
#define __Test__PhysicalSimulator__

#include <boost/noncopyable.hpp>
#include <boost/shared_ptr.hpp>
#include <ode/ode.h>
#include <ode/odecpp_collision.h>
#include <ode/odecpp.h>
#include <pcl/surface/gp3.h>
#include <vector>
#include "Device.h"

class PhysicalSimulator : private boost::noncopyable
{
private:
        // A ball consists of a rigid body and a geometry.
        struct Ball
        {
                dBody   body_;
                dSphere geom_;
        };

        dWorld          world_;
        dHashSpace      space_;
        dJointGroup     joint_group_;
        dGeomID         background_mesh_;
        dGeomID         object_mesh_;

        std::vector<boost::shared_ptr<Ball> > balls_;
        std::vector<boost::shared_ptr<dCylinder> > cylinders_;

        // the maximum number of contact points
        const static int MaxNumOfContacts = 1;
        std::vector<dContact> contacts_;
        
public:
        PhysicalSimulator();
        ~PhysicalSimulator();
        
        const std::vector<boost::shared_ptr<Ball> >& get_balls() const
        {
                return balls_;
        }
        
        const std::vector<boost::shared_ptr<dCylinder> >& get_cylinders() const
        {
                return cylinders_;
        }
        
        void create_world();
        void near_callback(void* data, dGeomID o1, dGeomID o2);
        void collide(void(*near_callback)(void* data, dGeomID o1, dGeomID o2));
        void create_balls();
        void create_cylinder(
                const dMatrix3& initial_rotation,
                dReal x,
                dReal y,
                dReal z,
                dReal radius,
                dReal length
        );
        void create_cylinders();
        
        void empty_join_group()
        {
                joint_group_.empty();
        }
        
        void step(dReal step)
        {
                world_.step(step);
        }
        
        void register_background_mesh(const boost::shared_ptr<Device::TriangularMesh>& mesh);
        void register_object_mesh(const boost::shared_ptr<Device::TriangularMesh>& mesh);
        
        void clear_balls();
        void clear_cylinders();

};
#endif /* defined(__Test__PhysicalSimulator__) */
