//
//  main.cpp
//  InteractiveObjects
//

#ifndef UNIT_TEST

#include <XnCppWrapper.h>
#include <stdexcept>
#include <boost/program_options.hpp>
#include <boost/timer.hpp>
#include <GLUT/GLUT.h>
#include "DeviceEnumerator.h"
#include "Evaluator.h"
#include "Viewer.h"
#include "Device.h"
#include "Context.h"

namespace po = boost::program_options;

// This application supports more than one viewer.
namespace
{
        Viewer viewer0;
//        Viewer viewer1;

        void near_callback0(void* data, dGeomID o1, dGeomID o2)
        {
                viewer0.near_callback(data, o1, o2);
        }
//        void near_callback1(void* data, dGeomID o1, dGeomID o2)
//        {
//                viewer1.near_callback(data, o1, o2);
//        }

        void display0()
        {
                viewer0.display(near_callback0);
        }

//        void display1()
//        {
//                viewer1.display(near_callback1);
//        }

        void on_key0(unsigned char key, int x, int y)
        {
                viewer0.on_key(key, x, y);
        }

//        void on_key1(unsigned char key, int x, int y)
//        {
//                viewer1.on_key(key, x, y);
//        }

        void idle()
        {
                glutSetWindow(viewer0.get_id());
                glutPostRedisplay();
//                glutSetWindow(viewer1.get_id());
//                glutPostRedisplay();
        }
        
        void usage()
        {
                std::cout << "usage:\n";
                std::cout << 'q' << " --- quit\n";
                std::cout << 'm' << " --- create triangular mesh\n";
                std::cout << 'p' << " --- display polygons\n";
                std::cout << 'o' << " --- enable object detection\n";
                std::cout << 'r' << " --- create rigid bodies\n";
                std::cout << std::endl;
        }
}

int main(int argc, char* argv[])
{
        try {
                po::options_description desc("options");
                desc.add_options()
                        ("help", "show this help")
                        ("width", po::value<int>()->default_value(320), "width of a window")
                        ("height", po::value<int>()->default_value(240), "height of a window")
                ;

                po::variables_map vm;
                po::store(po::parse_command_line(argc, argv, desc), vm);
                po::notify(vm);
                 
                if ( vm.count("help") ) {
                        std::cout << desc << std::endl;
                        return 1;
                }

                usage();
                Context::get_instance()->initialize();
                Context::get_instance()->set_global_mirror(true);
                
                DeviceEnumerator enumerator;
                enumerator.enumerate_production_trees(Context::get_instance());

                const int width = vm["width"].as<int>();
                const int height = vm["height"].as<int>();
                XnMapOutputMode map_mode;
                map_mode.nXRes = width;
                map_mode.nYRes = height;
                map_mode.nFPS = 30;
                
                enumerator.enumerate_existing_nodes(Context::get_instance(), map_mode);
                std::map<int, boost::shared_ptr<Device> >& devices = enumerator.get_devices();

                Context::get_instance()->start_generating_all();
                
                glutInit(&argc, argv);
                glutInitWindowSize(width, height);
                
                viewer0.initialize(map_mode, 0, devices[0]);
                glutDisplayFunc(display0);
                glutKeyboardFunc(on_key0);
               
//                viewer1.initialize(map_mode, 1, devices[1]);
//                glutDisplayFunc(display1);
//                glutKeyboardFunc(on_key1);

                glutIdleFunc(idle);
                glutMainLoop();
        } catch (const std::runtime_error& e) {
                std::cout << e.what() << std::endl;
        }
        return 0;
}
#else // UNIT_TEST

#define BOOST_AUTO_TEST_MAIN
#include <boost/test/unit_test.hpp>
#include <iostream>

BOOST_AUTO_TEST_CASE(TEST_main)
{
        std::cout << "TEST_main\n";
}
#endif // UNIT_TEST
